import { AxiosRequestConfig, Method } from 'axios';
import { API } from '../utils';

/* REQUEST_EXPORTS */

export const request = (
  method: Method,
  url: string,
  data: {} | null = null,
  headers: {} | null = null
) => {
  return API().request({ method, url, data, headers } as AxiosRequestConfig);
};

export interface Status {
  name: 'idle' | 'loading' | 'success' | 'failed';
}

export interface UpsertRelationshipsDataRequest<UR = null> {
  attributes?: UR | null;
}

export interface UpsertRelationshipsRequest<UR = null> {
  data?: UpsertRelationshipsDataRequest<UR> | null;
}

export interface UpsertDataRequest<UM, UR = null> {
  attributes: UM;
  relationships?: UpsertRelationshipsRequest<UR> | null;
}

export interface UpsertRequest<UM, UR = null> {
  data: UpsertDataRequest<UM, UR>;
}

export interface GetRelationshipsDataResponse {
  id: number;
  type: string;
}

export interface GetRelationshipsResponse {
  data: GetRelationshipsDataResponse[];
}

export interface GetIncludedResponse<R> {
  type?: string | null;
  id?: number | null;
  attributes: R;
}

export interface GetDataResponse<M, MR = null> {
  type?: string | null;
  id?: number | null;
  attributes: M;
  relationships?: MR | null;
}

export interface GetAllResponse<M, MR = null, R = null> {
  data: GetDataResponse<M, MR>[];
  included?: GetIncludedResponse<R>[] | null;
}

export interface GetOneResponse<M, MR = null, R = null> {
  data: GetDataResponse<M, MR>;
  included?: GetIncludedResponse<R>[] | null;
}

export interface DeleteAttributesResponse {
  success: boolean;
}

export interface DeleteDataResponse {
  attributes: DeleteAttributesResponse;
}

export interface DeleteResponse {
  data: DeleteDataResponse;
}

export interface ErrorAttributesResponse {
  status?: number | null;
  title?: string | null;
  detail?: string | null;
}

export interface ErrorResponse {
  errors?: ErrorAttributesResponse | null;
}
